// An implementation of deque using a chunklist
#ifndef _DEQUE_H_
#define _DEQUE_H_

#include "abstract_deque.h"

#include "dynamic_array_list.h"

template <typename T>
class Deque: public AbstractDeque<T>
{
public:
  // default constructor
  Deque();
  
  // copy constructor
  Deque(const Deque& x);

  // copy assignment
  Deque& operator=(Deque x);
  
  // destructor
  ~Deque();
  
  // return true is the queue is empty
  bool isEmpty();

  // enqueue (add) newEntry into the queue back
  // throw exception if not possible
  void enqueue_back(const T& item);

  // enqueue (add) newEntry into the queue front
  // throw exception if not possible
  void enqueue_front(const T& item);
  
  // dequeue (remove) newEntry from the queue front
  // throw exception if not possible
  void dequeue_front();

  // dequeue (remove) newEntry from the queue back
  // throw exception if not possible
  void dequeue_back();

  // return a copy of the item at the front of the queue
  // throw exception if not possible
  T peek_front();

  // return a copy of the item at the back of the queue
  // throw exception if not possible
  T peek_back();
  
private:

  void swap(Deque<T> &x, Deque<T> &y);

  // TODO
};

#include "deque.txx"

#endif // _DEQUE_H_
